__all__ = [
    "run_article_md_daily_notes",
    "run_article_tex_submit",
    "run_beamer_tex_weekly_reports",
    "run_search_for_screen",
    "run_search_for_files",
    "run_compare_after_search",
    "run_generate_c_yearly",
    "run_generate_j_e_weekly",
    "run_generate_j_weekly",
    "run_generate_j_monthly",
    "run_generate_j_yearly",
    # from pybibtexer
    "run_compare_bib_with_local",
    "run_compare_bib_with_zotero",
    "run_format_bib_to_save_by_entry_type",
    "run_format_bib_to_abbr_zotero_save",
    "run_replace_to_standard_cite_keys",
]

from .run_article_md import run_article_md_daily_notes
from .run_article_tex import run_article_tex_submit
from .run_beamer_tex import run_beamer_tex_weekly_reports
from .run_compare import run_compare_bib_with_local, run_compare_bib_with_zotero
from .run_format import run_format_bib_to_abbr_zotero_save, run_format_bib_to_save_by_entry_type
from .run_generate import (
    run_generate_c_yearly,
    run_generate_j_e_weekly,
    run_generate_j_monthly,
    run_generate_j_weekly,
    run_generate_j_yearly,
)
from .run_replace import run_replace_to_standard_cite_keys
from .run_search import run_compare_after_search, run_search_for_files, run_search_for_screen



import os
from typing import Any, Dict, List


def expand_path(path: str) -> str:
    """Expand user home directory and environment variables in path."""
    return os.path.expandvars(os.path.expanduser(path))


def expand_paths(*paths):
    # Expand and normalize file paths
    return [expand_path(path) for path in paths]


def build_base_options(
    include_publisher_list: List[str],
    include_abbr_list: List[str],
    exclude_publisher_list: List[str],
    exclude_abbr_list: List[str],
    path_conf_j_jsons: str,
) -> Dict[str, Any]:
    """
    Build options dictionary with common configuration.

    Args:
        include_publisher_list: List of publishers to include
        include_abbr_list: List of conference/journal abbreviations to include
        exclude_publisher_list: List of publishers to exclude
        exclude_abbr_list: List of conference/journal abbreviations to exclude
        path_conf_j_jsons: Base path for conferences/journals JSON files

    Returns:
        Dictionary containing configured options
    """
    path_conf_j_jsons = expand_path(path_conf_j_jsons)
    return {
        "include_publisher_list": include_publisher_list,
        "include_abbr_list": include_abbr_list,
        "exclude_publisher_list": exclude_publisher_list,
        "exclude_abbr_list": exclude_abbr_list,
        "full_json_c": os.path.join(path_conf_j_jsons, "conferences.json"),
        "full_json_j": os.path.join(path_conf_j_jsons, "journals.json"),
        "full_json_k": os.path.join(path_conf_j_jsons, "keywords.json"),
    }


def build_search_options(
    print_on_screen: bool, search_year_list: List[str], keywords_type: str, keywords_list_list: List[List[str]]
) -> Dict[str, Any]:
    """
    Build search options dictionary with common configuration.

    Args:
        print_on_screen: Whether to display results on screen
        search_year_list: List of years to filter search results
        keywords_type: Category name for search keywords
        keywords_list_list: Nested list of search keywords

    Returns:
        Dictionary containing configured search options
    """
    return {
        "print_on_screen": print_on_screen,
        "search_year_list": search_year_list,
        "keywords_dict": {keywords_type: keywords_list_list},
        "keywords_type_list": [keywords_type],
    }



from typing import Optional

from pybibtexer.tools import replace_to_standard_cite_keys

from ._base import build_base_options, expand_paths


def run_replace_to_standard_cite_keys(
    full_tex_md: str, full_bib: str, path_output: str, path_conf_j_jsons: str, options: Optional[dict] = None
) -> None:
    """
    Replace citation keys in LaTeX documents with standardized versions.

    Processes LaTeX and BibTeX files to normalize citation keys according to
    configuration standards, then outputs the results to the specified location.

    Args:
        options: Dictionary of configuration options for citation processing
        full_tex_md: Path to TeX or Markdown file containing citations
        full_bib: Path to the BibTeX bibliography file
        path_output: Output directory path for processed files
        path_conf_j_jsons: Path to journal configuration JSON files

    Returns:
        None: Results are written to the output directory
    """
    if options is None:
        options = {}

    # Expand and normalize file paths
    full_tex_md, full_bib, path_output = expand_paths(full_tex_md, full_bib, path_output)

    # Update options
    options_ = build_base_options([], [], [], [], path_conf_j_jsons)
    options_.update(options)

    replace_to_standard_cite_keys(full_tex_md, full_bib, path_output, options=options_)



from pathlib import Path

from pybibtexer.tools import format_bib_to_abbr_zotero_save_modes, format_bib_to_save_mode_by_entry_type

from ._base import build_base_options, expand_paths


def run_format_bib_to_save_by_entry_type(
    options: dict, need_format_bib: str, path_output: str, path_conf_j_jsons: str
) -> None:
    """
    Format a bibliography file by organizing entries according to their entry types and save the results.

    This function processes a bibliography file and reorganizes the entries based on their
    bibliographic entry types (e.g., article, conference, book, etc.), creating separate
    organized output files for better management and categorization.

    Args:
        options: Configuration dictionary for formatting behavior settings
        need_format_bib: Path to the bibliography file that needs to be formatted
        path_output: Output directory path where formatted bibliography files will be saved
        path_conf_j_jsons: Path to conference/journal configuration JSON files

    Returns:
        None: Formatted bibliography files are saved to the specified output directory
    """
    # Expand and normalize file paths
    need_format_bib, path_output = expand_paths(need_format_bib, path_output)

    # Update options
    options_ = build_base_options([], [], [], [], path_conf_j_jsons)
    options_.update(options)

    format_bib_to_save_mode_by_entry_type(Path(need_format_bib).stem, path_output, need_format_bib, options=options_)


def run_format_bib_to_abbr_zotero_save(
    options: dict, need_format_bib: str, path_output: str, path_conf_j_jsons: str
) -> None:
    """
    Format a bibliography file into three different modes: abbreviated, Zotero-compatible, and cleaned source.

    This function processes a bibliography file and generates three formatted versions:
    1. Abbreviated version: Journal/conference names are abbreviated according to standard rules
    2. Zotero-compatible version: Formatted specifically for Zotero reference management software
    3. Source preservation version: Maintains original content with consistent formatting and organization

    Args:
        options: Configuration dictionary for formatting behavior settings
        need_format_bib: Path to the bibliography file that needs to be formatted
        path_output: Output directory path where the three formatted bibliography files will be saved
        path_conf_j_jsons: Path to conference/journal configuration JSON files for abbreviation rules

    Returns:
        None: Three formatted bibliography files are saved to the specified output directory
    """
    # Expand and normalize file paths
    need_format_bib, path_output = expand_paths(need_format_bib, path_output)

    # Update options
    options_ = build_base_options([], [], [], [], path_conf_j_jsons)
    options_.update(options)

    format_bib_to_abbr_zotero_save_modes(need_format_bib, path_output, options=options_)



import copy
import os
import re
import shutil
import time
from typing import Any, Dict, List, Tuple

from pyadvtools import combine_content_in_list, delete_empty_lines_last_occur_add_new_line, read_list, write_list
from pybibtexer.bib.core import ConvertStrToLibrary
from pybibtexer.main.python_writers import PythonWriters

from .basic_input import BasicInput
from .pandoc_md_to import PandocMdTo


def batch_convert_citations(text):
    """
    Process all citations in the text, including multiple citations in one bracket.

    Example: [@ref1; @ref2] -> <sup>[@ref1](#ref1)</sup><sup>[@ref2](#ref2)</sup>
    """
    # Match all citation patterns within square brackets
    pattern = r"\[([^]]+)\]"

    def process_citation(match):
        citations = match.group(1)
        # Split multiple citations (support semicolon or comma separation)
        citation_list = re.split(r"[;,]", citations)

        result = []
        for citation in citation_list:
            citation = citation.strip()
            if citation.startswith("@"):
                cite_id = citation[1:]  # Remove the @ symbol
                result.append(f"[{citation}](#{cite_id.lower()})")
            else:
                # Keep non-citation content in original format
                result.append(f"[{citation}]")

        return "".join(result)

    return re.sub(pattern, process_citation, text)


class PythonRunMd(BasicInput):
    """Python markdown processing class.

    Args:
        options (Dict[str, Any]): Configuration options.

    Attributes:
        delete_temp_generate_md (bool): Whether to delete temporary generated markdown files. Defaults to True.
        add_reference_in_md (bool): Whether to add references in markdown. Defaults to True.
        add_bib_in_md (bool): Whether to add bibliography in markdown. Defaults to False.
        replace_cite_to_fullcite_in_md (bool): Whether to replace citations with full citations in markdown. Defaults to False.
        replace_by_basic_beauty_complex_in_md (str): Replace by basic, beauty, or complex format. Defaults to "beauty".
        display_basic_beauty_complex_references_in_md (str): Display basic, beauty, or complex references. Defaults to "beauty".
        add_anchor_in_md (bool): Whether add anchor in markdown. Defaults to False.
        details_to_bib_separator (str): Separator between <details> and bibliography content. Defaults to "\n".
    """

    def __init__(self, options: Dict[str, Any]) -> None:
        """Initialize PythonRunMd with configuration options.

        Args:
            options (Dict[str, Any]): Configuration options.
        """
        super().__init__(options)

        # for md
        self.final_output_main_md_name: str = options.get("final_output_main_md_name", "")
        self.delete_temp_generate_md: bool = options.get("delete_temp_generate_md", True)
        self.add_reference_in_md: bool = options.get("add_reference_in_md", True)
        self.add_bib_in_md: bool = options.get("add_bib_in_md", False)
        self.replace_cite_to_fullcite_in_md: bool = options.get("replace_cite_to_fullcite_in_md", False)
        self.replace_by_basic_beauty_complex_in_md: str = options.get("replace_by_basic_beauty_complex_in_md", "beauty")
        self.display_basic_beauty_complex_references_in_md: str = options.get(
            "display_basic_beauty_complex_references_in_md", "beauty"
        )
        self.add_anchor_in_md: bool = options.get("add_anchor_in_md", False)
        self.details_to_bib_separator: str = options.get("details_to_bib_separator", "\n")

        # for md
        self._pandoc_md_to = PandocMdTo(self.options)

        _options = {}
        _options["is_standardize_bib"] = False
        _options["is_display_implicit_comments"] = False
        _options.update(options)
        self._generate_library = ConvertStrToLibrary(_options)

    def special_operate_for_md(
        self,
        path_output: str,
        data_list_md: List[str],
        output_md_name: str,
        full_bib_for_abbr: str,
        full_bib_for_zotero: str,
        template_name: str = "article",
        generate_html: bool = False,
        generate_tex: bool = True,
    ) -> Tuple[List[str], List[str]]:
        """Perform special operations on markdown files.

        Args:
            path_output (str): Path to output directory.
            data_list_md (List[str]): List of markdown content lines.
            output_md_name (str): Name of output markdown file.
            full_bib_for_abbr (str): Path to abbreviated bibliography file.
            full_bib_for_zotero (str): Path to Zotero bibliography file.
            template_name (str, optional): Name of template to use. Defaults to "article".
            generate_html (bool, optional): Whether to generate HTML. Defaults to False.
            generate_tex (bool, optional): Whether to generate LaTeX. Defaults to True.

        Returns:
            Tuple[List[str], List[str]]: Tuple containing processed markdown and LaTeX content.
        """
        path_temp = os.path.join(path_output, "{}".format(time.strftime("%Y_%m_%d_%H_%M_%S")))
        if not os.path.exists(path_temp):
            os.makedirs(path_temp)

        # write original md content to temp file
        full_temp_md = os.path.join(path_temp, output_md_name)
        write_list(data_list_md, full_temp_md, "w", None, False)

        # pandoc md to md to update md content
        if read_list(full_bib_for_abbr, "r") and read_list(full_bib_for_zotero, "r"):
            data_list_md = self._special_operate_for_md(
                output_md_name, path_temp, full_bib_for_abbr, full_bib_for_zotero
            )
        elif os.path.exists(full_bib_for_abbr) and os.path.exists(full_bib_for_zotero):
            print(f"The content of bib: {full_bib_for_abbr} or {full_bib_for_zotero} is empty.")
        else:
            pass

        # main name
        main_name = self.final_output_main_md_name
        if len(main_name) == 0:
            main_name = output_md_name.split(".md")[0] + "_" + self.replace_by_basic_beauty_complex_in_md
            main_name = main_name + "_" + self.display_basic_beauty_complex_references_in_md + ".md"
        if main_name.lower() == output_md_name.lower():
            main_name = main_name.split(".md")[0] + "_.md"
        if main_name[-3:] != ".md":
            main_name = main_name + ".md"

        # write new generated md content to given file
        write_list(data_list_md, main_name, "w", path_output, False)

        # for generating html file from md file
        if data_list_md and generate_html:
            self._pandoc_md_to.pandoc_md_to_html(
                path_output, path_output, main_name, f'{main_name.split(".md")[0]}.html', True
            )

        # pandoc md to latex
        data_list_tex = []
        if generate_tex:
            n5 = "5_pandoc" + ".tex"
            data_list_tex = self._pandoc_md_to.pandoc_md_to_tex(template_name, path_temp, path_temp, output_md_name, n5)

        # delete temp path
        if self.delete_temp_generate_md:
            shutil.rmtree(path_temp)
        return data_list_md, data_list_tex

    def _special_operate_for_md(
        self, output_md_name: str, path_temp: str, full_bib_for_abbr: str, full_bib_for_zotero: str
    ) -> List[str]:
        """Perform special operations for markdown processing.

        Args:
            output_md_name (str): Name of output markdown file.
            path_temp (str): Path to temporary directory.
            full_bib_for_abbr (str): Path to abbreviated bibliography file.
            full_bib_for_zotero (str): Path to Zotero bibliography file.

        Returns:
            List[str]: List of processed markdown content lines.
        """
        # pandoc markdown to markdown
        n1 = "1_pandoc" + ".md"
        data_list_md = self._pandoc_md_to.pandoc_md_to_md(full_bib_for_abbr, path_temp, path_temp, output_md_name, n1)

        # use zotero bib to generate library
        bib_for_zotero = read_list(full_bib_for_zotero, "r")
        library = self._generate_library.generate_library(bib_for_zotero)

        _options = {}
        _options.update(self.options)
        _options["add_index_to_enties"] = False
        _python_writers = PythonWriters(_options)
        key_url_http_bib_dict = _python_writers.output_key_url_http_bib_dict(library)

        content_md = []
        if data_list_md and key_url_http_bib_dict:
            key_basic_dict, key_beauty_dict, key_complex_dict = self._pandoc_md_to.generate_key_data_dict(
                data_list_md, key_url_http_bib_dict
            )

            key_in_md = list(key_url_http_bib_dict.keys())

            # generate by replacing `- [@citation_key]` to `- [citation_key]`
            content = read_list(output_md_name, "r", path_temp)
            if self.replace_cite_to_fullcite_in_md:
                regex = re.compile(r"(\s*[-\+\*]\s*)\[@({})\]".format("|".join(key_in_md)))
                for i in range(len(content)):
                    if not (mch := regex.match(content[i])):
                        continue

                    content[i] = content[i].replace(mch.group(), mch.group(1) + "[" + mch.group(2) + "]")
            # add anchor
            if self.add_anchor_in_md:
                content = [batch_convert_citations(line) for line in content]
            n2 = "2_generate" + ".md"
            write_list(content, n2, "w", path_temp)

            # pandoc markdown to markdown
            n3 = "3_pandoc" + ".md"
            data_list_md = self._pandoc_md_to.pandoc_md_to_md(full_bib_for_abbr, path_temp, path_temp, n2, n3)

            # generate by replacing `- [citation_key]` to `- reference`
            if self.replace_cite_to_fullcite_in_md:
                regex = re.compile(r"(\s*)([-\+\*])(\s*)[\\]*\[({})[\\]*\]".format("|".join(key_in_md)))
                for i in range(len(data_list_md)):
                    if not (mch := regex.search(data_list_md[i])):
                        continue

                    space_one, b, space_two, cite_key = mch.groups()
                    if self.replace_by_basic_beauty_complex_in_md.lower() == "basic":
                        temp_list = copy.deepcopy(key_basic_dict[cite_key])
                    elif self.replace_by_basic_beauty_complex_in_md.lower() == "complex":
                        temp_list = copy.deepcopy(key_complex_dict[cite_key])
                    else:
                        temp_list = copy.deepcopy(key_beauty_dict[cite_key])

                    temp = "".join(self._special_format(temp_list, space_one, space_two))
                    data_list_md[i] = data_list_md[i].replace(mch.group(), space_one + b + space_two + temp.strip())
            n4 = "4_generate" + ".md"
            write_list(data_list_md, os.path.join(path_temp, n4), "w")

            # obtain footnote part (in the last part of the contents)
            main_part, last_part = [], []
            main_part_flag, last_part_flag = True, True
            for line_index in range(len(data_list_md)):
                if main_part_flag and re.match(r"#+ [\"']?References[\"']?\s[\[{]", data_list_md[line_index]):
                    main_part_flag = False
                    main_part = delete_empty_lines_last_occur_add_new_line(data_list_md[:line_index])
                if last_part_flag and re.match(r"^\[\^1]: ", data_list_md[line_index]):
                    last_part_flag = False
                    last_part = delete_empty_lines_last_occur_add_new_line(data_list_md[line_index:])

            if main_part_flag:
                main_part = delete_empty_lines_last_occur_add_new_line(data_list_md)
            if self.add_reference_in_md:
                main_part.append("\n## References\n")
            if len(last_part) > 0:
                last_part.insert(0, "[//]: (Footnotes)\n\n")
                last_part.append("\n")

            # for bib
            bib_in_md = []
            if self.add_bib_in_md:
                temp_c = combine_content_in_list([key_url_http_bib_dict[k][2] for k in key_in_md])
                bib_in_md = combine_content_in_list(
                    [
                        ["## Bibliography\n\n"],
                        [f"<details>{self.details_to_bib_separator}"],
                        ["```\n"],
                        temp_c,
                        ["```\n"],
                        ["</details>\n"],
                    ]
                )

            # Generate basic/beauty/complex markdown content
            if dct := eval(f"key_{self.display_basic_beauty_complex_references_in_md}_dict"):
                content_md = self._generate_content_md(dct, key_in_md, main_part, last_part, bib_in_md)
        return content_md

    @staticmethod
    def _special_format(temp_list: List[str], space_one: str, space_two: str) -> List[str]:
        """Apply special formatting for alignment.

        Args:
            temp_list (List[str]): List of strings to format.
            space_one (str): First space string.
            space_two (str): Second space string.

        Returns:
            List[str]: Formatted list of strings.
        """
        for j in range(len(temp_list) - 1):
            if temp_list[j][-1] == "\n":
                temp_list[j + 1] = (space_one + " " + space_two) + temp_list[j + 1]
        return temp_list

    def _generate_content_md(
        self,
        key_basic_beauty_complex_dict: Dict[str, List[str]],
        key_in_md_tex: List[str],
        main_part: List[str],
        last_part: List[str],
        bib_in_md: List[str],
    ) -> List[str]:
        """Generate markdown content from various components.

        Args:
            key_basic_beauty_complex_dict (Dict[str, List[str]]): Dictionary of formatted references.
            key_in_md_tex (List[str]): List of citation keys in markdown/LaTeX.
            main_part (List[str]): Main content part.
            last_part (List[str]): Last content part.
            bib_in_md (List[str]): Bibliography content for markdown.

        Returns:
            List[str]: Generated markdown content.
        """
        temp_b = []
        if self.add_reference_in_md:
            temp_b = combine_content_in_list([key_basic_beauty_complex_dict[k] for k in key_in_md_tex], ["\n"])
        content_md = combine_content_in_list([main_part, ["\n"], temp_b, last_part, bib_in_md])
        return content_md



import os
from typing import Any, Dict

from pyadvtools import read_list
from pybibtexer.main import BasicInput as BasicInputInPyBibtexer


class BasicInput(BasicInputInPyBibtexer):
    """Basic input class for handling bibliography and template configurations.

    Args:
        options (Dict[str, Any]): Configuration options.

    Attributes:
        full_csl_style_pandoc (str): Full path to CSL style for pandoc.
        full_tex_article_template_pandoc (str): Full path to tex article template for pandoc.
        full_tex_beamer_template_pandoc (str): Full path to tex beamer template for pandoc.
        article_template_tex (List[str]): Article template for LaTeX.
        article_template_header_tex (List[str]): Article template header for LaTeX.
        article_template_tail_tex (List[str]): Article template tail for LaTeX.
        beamer_template_header_tex (List[str]): Beamer template header for LaTeX.
        beamer_template_tail_tex (List[str]): Beamer template tail for LaTeX.
        math_commands_tex (List[str]): LaTeX math commands.
        usepackages_tex (List[str]): LaTeX usepackages.
        handly_preamble (bool): Whether to handle preamble manually.
        options (Dict[str, Any]): Configuration options.
    """

    def __init__(self, options: Dict[str, Any]) -> None:
        """Initialize BasicInput with configuration options.

        Args:
            options (Dict[str, Any]): Configuration options dictionary.
        """
        super().__init__(options)

        current_dir = os.path.dirname(os.path.abspath(__file__))
        self._path_templates = os.path.join(os.path.dirname(current_dir), "data", "templates")

        # main
        self._initialize_pandoc_md_to(options)
        self._initialize_python_run_tex(options)

        self.options = options

    def _initialize_pandoc_md_to(self, options: Dict[str, Any]) -> None:
        """Initialize pandoc markdown to other formats configuration.

        Args:
            options (Dict[str, Any]): Configuration options.
        """
        csl_name = options.get("csl_name", "apa-no-ampersand")
        if not isinstance(csl_name, str):
            csl_name = "apa-no-ampersand"
        self.full_csl_style_pandoc = os.path.join(self._path_templates, "csl", f"{csl_name}.csl")
        if not os.path.exists(self.full_csl_style_pandoc):
            self.full_csl_style_pandoc = os.path.join(self._path_templates, "csl", "apa-no-ampersand.csl")

        self.full_tex_article_template_pandoc = os.path.join(self._path_templates, "tex", "eisvogel.latex")
        self.full_tex_beamer_template_pandoc = os.path.join(self._path_templates, "tex", "eisvogel.beamer")

        self.article_template_tex = self._try_read_list("tex", "Article.tex")

    def _initialize_python_run_tex(self, options: Dict[str, Any]) -> None:
        """Initialize Python LaTeX processing configuration.

        Args:
            options (Dict[str, Any]): Configuration options.
        """
        self.article_template_header_tex = self._try_read_list("tex", "Article_Header.tex")
        self.article_template_tail_tex = self._try_read_list("tex", "Article_Tail.tex")
        self.beamer_template_header_tex = self._try_read_list("tex", "Beamer_Header.tex")
        self.beamer_template_tail_tex = self._try_read_list("tex", "Beamer_Tail.tex")
        self.math_commands_tex = self._try_read_list("tex", "math_commands.tex")
        self.usepackages_tex = self._try_read_list("tex", "Style.tex")

        # handly preamble
        self.handly_preamble = options.get("handly_preamble", False)
        if self.handly_preamble:
            self.article_template_header_tex, self.article_template_tail_tex = [], []
            self.beamer_template_header_tex, self.beamer_template_tail_tex = [], []
            self.math_commands_tex, self.usepackages_tex = [], []

    def _try_read_list(self, folder_name: str, file_name: str):
        """Try to read a list from a file in the templates directory.

        Args:
            folder_name (str): Name of the folder in templates directory.
            file_name (str): Name of the file to read.

        Returns:
            List[str]: List of lines from the file, or empty list if file cannot be read.
        """
        path_file = os.path.join(self._path_templates, folder_name, file_name)

        try:
            data_list = read_list(path_file)
        except Exception as e:
            print(e)
            data_list = []
        return data_list
